<?php
/**
 * @author : Jegtheme
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

Class JNews_Translation_Option
{
    /**
     * @var JNews_Translation_Option
     */
    private static $instance;

    /**
     * @return JNews_Translation_Option
     */
    public static function getInstance()
    {
        if (null === static::$instance)
        {
            static::$instance = new static();
        }
        return static::$instance;
    }

    private function __construct()
    {
        if(is_admin())
        {
            add_filter('jnews_admin_menu', array($this, 'admin_menu'));
            add_filter('jnews_admin_slug', array($this, 'admin_slug'));
            add_action('admin_notices', array( $this, 'admin_notice' ));
        }

        $this->theme_option();
    }

    public function admin_notice()
    {
        $slug = apply_filters('jnews_get_admin_slug', '');

        if(isset($_REQUEST['page']) && $_REQUEST['page'] === $slug['translation'])
        {
            // polylang
            if(function_exists('pll_current_language'))
            {
                $current_language = pll_current_language('name');
                if(empty($current_language)) $current_language = pll_default_language('name');

                printf(
                    '<div class="updated"><p>%s : <strong>%s</strong></p></div>',
                    esc_html__('Frontend Translation Language', 'jnews'),
                    $current_language
                );
            }

            // wpml
            if ( defined('ICL_SITEPRESS_VERSION') ) 
            {
                if ( defined('ICL_LANGUAGE_NAME') ) 
                {
                    printf(
                        '<div class="updated"><p>%s : <strong>%s</strong></p></div>',
                        esc_html__('Frontend Translation Language', 'jnews'),
                        ICL_LANGUAGE_NAME
                    );
                }
            }
        }
    }

    public function admin_slug($slug)
    {
        $translation_slug = array(
            'translation' => 'jnews_translation'
        );

        return array_merge($translation_slug, $slug);
    }

    public function admin_menu($menu)
    {
        $slug = apply_filters('jnews_get_admin_slug', '');

        $translation_menu = array(
            array(
                'title' => esc_html__('Translate Frontend', 'jnews-front-translation'),
                'menu' => esc_html__('Translate Frontend', 'jnews-front-translation'),
                'slug' => $slug['translation'],
                'action' => false,
                'priority' => 54,
                'show_on_menu' => false
            )
        );

        return array_merge($menu, $translation_menu);
    }

    public function translation_slug()
    {
        $adminslug = apply_filters('jnews_get_admin_slug', '');
        return is_array($adminslug) && isset($adminslug['translation']) ? $adminslug['translation'] : '';
    }

    public function theme_option()
    {
        if(class_exists('VP_Option'))
        {
            $dashboard_option = $this->dashboard_option();
            $translation_slug = $this->translation_slug();

            $this->option =
                new VP_Option(array(
                    'is_dev_mode' => false,
                    'option_key' => 'jnews_translate',
                    'page_slug' => $translation_slug,
                    'menu_page' => 'jnews',
                    'template' => $dashboard_option,
                    'use_auto_group_naming' => true,
                    'use_util_menu' => true,
                    'minimum_role' => 'edit_theme_options',
                    'layout' => 'fixed',
                    'page_title' => 'Translate Frontend',
                    'menu_label' => 'Translate Frontend',
                    'priority' => 54
                ));
        }
    }

    public function dashboard_option()
    {
        $self = $this;

        add_filter('jnews_admin_translate_option', function ($option) use ($self) {
            $translation_setting = include JNEWS_FRONT_TRANSLATION_DIR . 'options/translation-setting.php';
            $option = $self->merge_option($option, $translation_setting);

            $translation = include JNEWS_FRONT_TRANSLATION_DIR . 'options/translation.php';
            $option = $self->merge_option($option, $translation);

            return $option;
        }, 10);

        return array(
            'title' => 'Frontend Translation',
            'logo' => '',
            'menus' => apply_filters('jnews_admin_translate_option', array()),
        );
    }

    public function merge_option($option, $newoption)
    {
        if (empty($option)) {
            return array($newoption);
        } else {
            return array_merge($option, array($newoption));
        }
    }
}
